searchData={"items":[{"type":"module","title":"EctoRanked","doc":"This module provides support for automatic ranking of your Ecto models.","ref":"EctoRanked.html"},{"type":"function","title":"EctoRanked.set_rank/2","doc":"Updates the given changeset with the appropriate ranking, and updates/ranks\nthe other items in the list as necessary.","ref":"EctoRanked.html#set_rank/2"},{"type":"function","title":"Options - EctoRanked.set_rank/2","doc":"* `:rank` - the field to store the actual ranking in. Defaults to `:rank`\n* `:position` - the field to use for positional changes. Defaults to `:position`\n* `:scope` - the field(s) to scope all rankings to. Defaults to `nil` (no scoping).\n* `:prefix` - the prefix to run all queries on (e.g. the schema path in Postgres). Defaults to `nil` (no prefix).","ref":"EctoRanked.html#set_rank/2-options"},{"type":"module","title":"EctoRanked.Utils","doc":"Provides utilities for EctoRanked","ref":"EctoRanked.Utils.html"},{"type":"function","title":"EctoRanked.Utils.ceiling/1","doc":"Returns the first integer greater or equal than the provided value","ref":"EctoRanked.Utils.html#ceiling/1"},{"type":"function","title":"Examples - EctoRanked.Utils.ceiling/1","doc":"iex> EctoRanked.Utils.ceiling(2.4)\n    3","ref":"EctoRanked.Utils.html#ceiling/1-examples"},{"type":"extras","title":"EctoRanked","doc":"# EctoRanked\n\nThis package adds automatic ranking to your Ecto models. It's heavily based on\nthe Rails [ranked-model](https://github.com/mixonic/ranked-model) gem.","ref":"readme.html"},{"type":"extras","title":"Installation - EctoRanked","doc":"The package can be installed by adding `ecto_ranked` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:ecto_ranked, \"~> 0.6.0\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - EctoRanked","doc":"To get started:\n\n- `import EctoRanked`\n- Add a `:rank` integer field to your model (NOTE: Setting a unique index on this column may cause issues depending on your database platform)\n- Call `set_rank()` in your changeset\n- Optionally, add a virtual `:position` field (with a type of `:any`) so you can move items in your list.\n\n```elixir\ndefmodule MyApp.Item do\n  use MyApp.Web, :model\n  import EctoRanked\n\n  schema \"items\" do\n    field :rank, :integer\n    field :position, :any, virtual: true\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> cast(params, [:position])\n    |> set_rank()\n  end\nend\n```\n\nIf you need to use field names other than `:rank` and `:position`, you can pass those as options to `set_rank`:\n\n```elixir\ndefmodule MyApp.Item do\n  use MyApp.Web, :model\n  import EctoRanked\n\n  schema \"items\" do\n    field :my_ranking_field, :integer\n    field :my_position_field, :any, virtual: true\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> cast(params, [:my_position_field])\n    |> set_rank(rank: :my_ranking_field, position: :my_position_field)\n  end\nend\n```\n\nIf you'd like to scope your ranking to a certain field (e.g. an association, string field, etc.),\njust add a `:scope` argument to `set_rank`:\n\n```elixir\ndefmodule MyApp.Item do\n  use MyApp.Web, :model\n  import EctoRanked\n\n  schema \"items\" do\n    field :rank, :integer\n    field :position, :any, virtual: true\n    belongs_to :parent, MyApp.Parent\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> cast(params, [:position])\n    |> set_rank(scope: :parent_id)\n  end\nend\n```\n\nYou can scope across multiple fields:\n\n```elixir\nstruct\n|> cast(params, [:position, :parent_id, :category])\n|> set_rank(scope: [:parent_id, :category])\n```\n\nScopes are optional by default, meaning within a single table you might have some records scoped against a value, and some records not scoped at all (when the scoped value receives a `nil` value). Those unscoped records are treated as their own global scope. If you want to ensure a scope is always provided, you can use the `scope_required` option, which is effectively the same as adding your own `validate_required/3` call:\n\n```elixir\nstruct\n|> cast(params, [:position, :parent_id, :category])\n|> set_rank(scope: :parent_id, scope_required: true)\n```\n\nYou can even have multiple rankings that sort independently of each other (e.g. a scoped one and a global one, or multiple global ones):\n\n```elixir\nstruct\n|> cast(params, [:local_position, :global_position, :parent_id])\n|> set_rank(rank: :scoped_rank, position: :scoped_position, scope: :parent_id)\n|> set_rank(rank: :global_rank, position: :global_position)\n```\n\nPosition is a write-only virtual attribute that's meant for placing an item at\na specific rank. By default the `position` attribute will be `nil` but you can\ncalculate it on demand:\n\n```elixir\ndef compute_positions(items \\\\ []) do\n  for {item, i} <- Enum.with_index(items) do\n    %{item | position: i}\n  end\nend\n\nItem\n|> order_by([:rank])\n|> Repo.all()\n|> Item.compute_positions()\n```","ref":"readme.html#usage"},{"type":"extras","title":"Documentation - EctoRanked","doc":"Documentation can be found at [https://hexdocs.pm/ecto_ranked](https://hexdocs.pm/ecto_ranked).","ref":"readme.html#documentation"},{"type":"extras","title":"Thanks - EctoRanked","doc":"- Everyone who contributed to [ranked-model](https://github.com/mixonic/ranked-model/graphs/contributors), of which this package is a rough clone.\n- [EctoOrdered](https://github.com/zovafit/ecto-ordered), which provided a great starting point.","ref":"readme.html#thanks"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}